using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing.Design;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;
using Dapfor.Net.Grid.Tutorial.BusinessData.CAC40;
using Dapfor.Net.Grid.Tutorial.Editors;

namespace Dapfor.Net.Grid.Tutorial.BusinessData.OrderBook
{
    public class Order : INotifyPropertyChanged
    {
        #region Fields

        private readonly DateTime _creationDate;
        private readonly ThreadSafeBindingList<Deal> _deals = new ThreadSafeBindingList<Deal>();
        private readonly string _id;
        private readonly Instrument _instrument;
        private readonly Strategy _strategy;
        private DateTime _lastChangeTimeStamp;
        private double _price;
        private long _quantity;
        private Status _status;
        private Way _way;

        #endregion

        public Order(string id, Way way, double price, long quantity, Instrument instrument, Strategy strategy)
        {
            _id = id;
            _way = way;
            _price = price;
            _quantity = quantity;
            _strategy = strategy;
            _instrument = instrument;
            _creationDate = _lastChangeTimeStamp = DateTime.Now;

            _deals.ListChanged += delegate
            {
                _lastChangeTimeStamp = DateTime.Now;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("LastChangeTimeStamp"));
                }
            };
        }

        #region Public properties

        [HierarchicalField]
        public IList<Deal> Deals
        {
            get { return _deals; }
        }

        public string Id
        {
            get { return _id; }
        }

        [CompositeField]
        public Instrument Instrument
        {
            get { return _instrument; }
        }

        public Strategy Strategy
        {
            get { return _strategy; }
        }

        [Field("TimeStamp")]
        public DateTime CreationDate
        {
            get { return _creationDate; }
        }

        public DateTime LastChangeTimeStamp
        {
            get { return _lastChangeTimeStamp; }
        }

        public Way Way
        {
            get { return _way; }
            set
            {
                UpdateChangeTimeStamp();
                if (!Equals(_way, value))
                {
                    _way = value;
                    FirePropertyChanged("Way");
                }
            }
        }

        [DoubleFormat(3)]
        public double Price
        {
            get { return _price; }
            set
            {
                UpdateChangeTimeStamp();
                if (_price != value)
                {
                    _price = value;
                    FirePropertyChanged("Price");
                }
            }
        }

        public long Quantity
        {
            get { return _quantity; }
            set
            {
                UpdateChangeTimeStamp();
                if (_quantity != value)
                {
                    _quantity = value;
                    FirePropertyChanged("Quantity");
                }
            }
        }

        [Editor(typeof(OrderStatusEditor), typeof(UITypeEditor))]
        public Status Status
        {
            get { return _status; }
            set
            {
                if (!Equals(_status, value))
                {
                    _status = value;
                    FirePropertyChanged("Status");
                }
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods

        private void UpdateChangeTimeStamp()
        {
            _lastChangeTimeStamp = DateTime.Now;
            FirePropertyChanged("LastChangeTimeStamp");
        }

        private void FirePropertyChanged(string id)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(id));
            }
        }

        #endregion
    }
}