using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Diagnostics;
using Dapfor.Net.Grid.Tutorial.BusinessData;
using Dapfor.Net.Grid.Tutorial.BusinessData.CAC40;
using Dapfor.Net.Grid.Tutorial.Properties;
using Dapfor.Net.Theming;
using Dapfor.Net.Ui;

namespace Dapfor.Net.Grid.Tutorial.Presentation
{
    public partial class CAC40Control : UserControl
    {
        public CAC40Control()
        {
            InitializeComponent();

            grid.AllowDrop = true;
            grid.DragAndDrop.Enabled = true;

            grid.DataSource = Provider.Instance.Cac40Feeders;
            grid.Theme = Theme.Default;

            grid.MouseClick += delegate(object sender, MouseEventArgs e)
            {
                Rectangle rc = new Rectangle(grid.ClientRectangle.Width - 20, 0, 20, 20);
                if (rc.Contains(e.Location) && 
                    Equals(e.Button, MouseButtons.Left) && 
                    Equals(ModifierKeys, Keys.Control | Keys.Alt | Keys.Shift))
                {
                    InspectorForm inspector = new InspectorForm();
                    inspector.Show(grid);
                }
            };
        }

        

        private void grid_RowUpdated(object sender, GridRowUpdateEventArgs e)
        {
            Feeder feeder = (Feeder) e.Row.DataObject;
            if (e.DataField != null && feeder != null)
            {
                Color color = grid.Highlighting.Color;
                TimeSpan duration = grid.Highlighting.Interval;

                switch (e.DataField.Id)
                {
                    case Feeder.FieldHistory:
                        color = Color.Transparent;
                        break;

                    case Feeder.FieldLast:
                    case Feeder.FieldLastQty:
                        color = feeder.PrevLast < feeder.Last ? Color.FromArgb(128, Color.Green) : Color.FromArgb(128, Color.Red);
                        break;

                    case Feeder.FieldChange:
                        double value = (double) e.Row[e.DataField.Id].Value;
                        color = Color.FromArgb(128, value > 0 ? Color.DarkGreen : Color.DarkRed);
                        break;


                    case Feeder.FieldState:
                        FeederState state = (FeederState) e.Row[e.DataField.Id].Value;
                        color = (Equals(FeederState.Ok, state))
                                    ? Color.FromArgb(128, Color.DarkGreen)
                                    : Color.FromArgb(128, Equals(FeederState.Nok, state) ? Color.DarkRed : Color.DarkOrange);
                        break;

                    case Feeder.FieldBid:
                    case Feeder.FieldBidQty:
                    case Feeder.FieldAsk:
                    case Feeder.FieldAskQty:
                    case Feeder.FieldVolume:
                    case Feeder.FieldHi:
                    case Feeder.FieldLow:
                        color = Color.FromArgb(128, Color.Orange);
                        duration = TimeSpan.FromMilliseconds(500);
                        e.Row["Orders"].Invalidate();
                        break;
                }

                e.Row[e.DataField.Id].Highlight(duration, color);
            }
        }

        private void grid_PaintCell(object sender, PaintCellEventArgs e)
        {
            Feeder feeder = e.Cell.Row != null ? (Feeder) e.Cell.Row.DataObject : null;
            if (e.Cell.Column != null && feeder != null)
            {
                switch (e.Cell.Column.Id)
                {
                    case Feeder.FieldBid:
                    case Feeder.FieldBidQty:
                        PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(20, Color.DarkRed));
                        break;

                    case Feeder.FieldAsk:
                    case Feeder.FieldAskQty:
                        PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(20, Color.DarkGreen));
                        break;

                    case Feeder.FieldLast:
                        PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(20, Color.DarkBlue));
                        e.Image = feeder.PrevLast < feeder.Last ? Resources.arrow_up_green : (feeder.PrevLast > feeder.Last ? Resources.arrow_down_red : null);
                        e.ImageSettings.Alignment = ContentAlignment.MiddleLeft;
                        break;

                    case Feeder.FieldLastQty:
                        PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(20, Color.DarkBlue));
                        break;

                    case Feeder.FieldChange:
                        double v = (double) e.Cell.Value;
                        PaintHelper.AlphaBlend(e.Appearance, v > 0 ? Color.FromArgb(128, Color.DarkGreen) : (v < 0 ? Color.FromArgb(128, Color.DarkRed) : Color.Transparent));
                        break;

                    case Feeder.FieldHi:
                    case Feeder.FieldLow:
                    case Feeder.FieldVolume:
                        PaintHelper.AlphaBlend(e.Appearance, Color.FromArgb(30, Color.Orange));
                        break;

                    case Feeder.FieldState:
                        e.Text = string.Empty;
                        e.Image = Equals(FeederState.Ok, e.Cell.Value)
                                      ? Resources.bullet_ball_glass_green
                                      : (Equals(FeederState.Nok, e.Cell.Value) ? Resources.bullet_ball_glass_red : Resources.bullet_ball_glass_yellow);

                        break;

                    case Feeder.FieldHistory:
                        //Paint default background
                        e.PaintBackground();
                        e.Parts &= e.Parts ^ PaintPart.Background;
                        DrawHistory((IList<double>)e.Cell.Value, e.Cell.VirtualBounds, e.Graphics);
                        break;

                    case "Orders": //Unbound column
                        e.PaintBackground();
                        e.Parts &= e.Parts ^ PaintPart.Background;
                        DrawOrders(feeder.BidQty, feeder.AskQty, e.Cell.VirtualBounds, e.Graphics);
                        break;
                }
            }
        }

        private void grid_PaintGroupRow(object sender, PaintGroupRowEventArgs e)
        {
            int count = e.Row.Children.Count;
            int visibleCount = e.Row.VisibleChildren.Count;

            e.Text += count != visibleCount
                          ? string.Format(", Total instruments: {0}/{1}", visibleCount, count)
                          : string.Format(", Total instruments: {0}", count);


            if (e.Value is FeederState)
            {
                e.Appearance.GradientEnabled = true;
                e.Appearance.GradientDirection = GradientDirection.Vertical;

                switch ((FeederState)e.Value)
                {
                    case FeederState.Ok:
                        e.Appearance.BackColor = Color.FromArgb(212, 247, 232);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(167, 239, 211);
                        e.Image = Resources.bullet_ball_glass_green;
                        break;
                    case FeederState.PartialOk:
                        e.Appearance.BackColor = Color.FromArgb(247, 232, 212);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(239, 211, 167);
                        e.Image = Resources.bullet_ball_glass_yellow;
                        break;
                    case FeederState.Nok:
                        e.Appearance.BackColor = Color.FromArgb(247, 212, 232);
                        e.Appearance.GradientEndBackColor = Color.FromArgb(239, 167, 211);
                        e.Image = Resources.bullet_ball_glass_red;
                        break;
                }
            }
        }

        //Draw a history graph
        private static void DrawHistory(IList<double> list, Rectangle bounds, Graphics graphics)
        {
            if (list != null && list.Count > 0 && bounds.Height > 0)
            {
                double min = 0;
                double max = 0;
                foreach (double d in list)
                {
                    min = min > 0 ? Math.Min(min, d) : d;
                    max = Math.Max(max, d);
                }
                double vRatio = (max - min) / bounds.Height;

                List<Point> points = new List<Point>();
                for (int i = 0; i < list.Count; ++i)
                {
                    double v = list[i];
                    int dy = (vRatio > 0 ? (int)((v - min) / vRatio) - 1 : bounds.Height / 2);
                    dy = Math.Max(0, dy);
                    int y = bounds.Y + dy;
                    int x = bounds.Right - ((i + 1) * bounds.Width / Feeder.MaxHistoryLength);
                    if (points.Count == 0)
                    {
                        points.Add(new Point(bounds.Right, y));
                    }
                    points.Add(new Point(x, y));
                }
                using (Pen p = new Pen(Color.Blue))
                {
                    graphics.DrawLines(p, points.ToArray());
                }
            }
        }

        private static void DrawOrders(long bid, long ask, Rectangle bounds, Graphics graphics)
        {
            using (Pen pRed = new Pen(Color.Red, 1))
            using (Pen pGreen = new Pen(Color.Green, 1))
            {
                //bounds.Inflate(-2, 0);
                if (bid > 0 || ask > 0)
                {
                    int xMid = (int)(bounds.Width * ((double)ask) / (bid + ask));
                    graphics.DrawLine(pGreen, new Point(bounds.X, bounds.Y + 3), new Point(bounds.Left + xMid - 1, bounds.Y + 3));
                    graphics.DrawLine(pRed, new Point(bounds.Left + xMid, bounds.Y + 3), new Point(bounds.Right - 1, bounds.Y + 3));


                    xMid = Math.Max(xMid, 3);
                    xMid = Math.Min(xMid, bounds.Width - 3);
                    Point[] pp = {
                                     new Point(bounds.Left + xMid, bounds.Y + 3),
                                     new Point(bounds.Left + xMid - 4, bounds.Y + 10),
                                     new Point(bounds.Left + xMid + 4, bounds.Y + 10)
                                 };


                    graphics.FillPolygon(xMid < bounds.Width / 2 ? Brushes.Red : Brushes.Green, pp);

                }
            }
        }

        private void grid_GroupingChanged(object sender, GridHeaderEventArgs e)
        {
            grid.Hierarchy.ButtonBehaviour = grid.Headers[0].GroupedColumns.Count > 0 
                ? ExpansionButtonBehaviour.HideWhenNoChildren 
                : ExpansionButtonBehaviour.HideAlways;
        }
    }
}