using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Dapfor.Net.Grid.Tutorial.BusinessData;
using Dapfor.Net.Grid.Tutorial.BusinessData.Currencies;
using Dapfor.Net.Theming;
using Dapfor.Net.Ui;
using StringFormat = Dapfor.Net.Formats.StringFormat;

namespace Dapfor.Net.Grid.Tutorial.Presentation
{
    public partial class CurrencyControl : UserControl
    {
        private readonly Color _hotColor = Color.FromArgb(40, Color.Blue);
        private readonly Color _mirroirHotColor = Color.FromArgb(40, Color.Orange);

        public CurrencyControl()
        {
            InitializeComponent();

            Header header = Header.FromDataType(typeof (CurrencyRate));
            grid.Headers.Add(header);
            foreach (Column column in header.Columns)
            {
                column.Width = 40;
                column.Format = new StringFormat("0.###");
                column.Sortable = false;
                column.Resizable = false;
                column.Movable = false;
            }
            header.Configurable = false;
            

            //Bind grid to the collection of CurrencyRate
            grid.DataSource = new List<CurrencyRate>(Provider.Instance.CurrencyRates.Values);
            grid.Theme = Theme.Default;
        }

        private void grid_PaintRow(object sender, PaintRowEventArgs e)
        {
            //Prevent from drawing the image in row selector
            e.Parts &= e.Parts ^ PaintPart.RowSelectorImages;

            //Do dafault drawing
            e.PaintAll();
            e.Handled = true;

            //Draw text in the row selector
            if (e.Row != null)
            {
                using (System.Drawing.StringFormat sf = new System.Drawing.StringFormat())
                {
                    sf.Alignment = StringAlignment.Center;
                    e.Graphics.DrawString(((CurrencyRate) e.Row.DataObject).MainCurrency, grid.Headers[0].Appearance.ColumnCaptionFont, Brushes.Black, e.RowSelectorBounds, sf);
                }
            }
        }

        private void grid_RowUpdated(object sender, GridRowUpdateEventArgs e)
        {
            CurrencyRate rate = (CurrencyRate) e.Row.DataObject;
            string currency = e.DataField.Id;
            Color color = rate[currency].Rate > rate[currency].PrevRate ? Color.FromArgb(60, Color.Green) : Color.FromArgb(60, Color.Red);
            e.Row[currency].Highlight(TimeSpan.FromMilliseconds(500), color);
        }

        private void grid_PaintCell(object sender, PaintCellEventArgs e)
        {
            if(e.Cell.Column != null)
            {
                Color color = e.Appearance.BackColor;
                
                //Hot column and row
                Point pt = e.Grid.PointToClient(Cursor.Position);
                if (e.Cell.Row.Bounds.Contains(pt))
                {
                    color = PaintHelper.AlphaBlend(color, _hotColor);
                }
                if (pt.X > e.Cell.VisibleBounds.Left && pt.X < e.Cell.VisibleBounds.Right && e.Grid.ClientRectangle.Contains(pt) && e.Grid.HitTests.CellTest(pt) != null)
                {
                    color = PaintHelper.AlphaBlend(color, _hotColor);
                }

                //Mirroir column and row 
                Cell mirorredCell = GetMirroredCell(e.Grid.HitTests.CellTest(pt));
                if (mirorredCell != null && mirorredCell.Row.VisibleIndex == e.Cell.Row.VisibleIndex)
                {
                    color = PaintHelper.AlphaBlend(color, _mirroirHotColor);
                }
                if (mirorredCell != null && mirorredCell.Column.VisibleIndex == e.Cell.Column.VisibleIndex)
                {
                    color = PaintHelper.AlphaBlend(color, _mirroirHotColor);
                }



                //Direct and inverse rates
                color = e.Cell.Column.VisibleIndex == e.Cell.Row.VisibleIndex
                                  ? PaintHelper.AlphaBlend(color, Color.FromArgb(40, Color.Blue))
                                  : (e.Cell.Column.VisibleIndex > e.Cell.Row.VisibleIndex
                                         ? PaintHelper.AlphaBlend(color, Color.FromArgb(30, Color.Orange))
                                         : PaintHelper.AlphaBlend(color, Color.FromArgb(30, Color.DarkGreen)));

                e.Appearance.BackColor = color;

                //A frame around hot cell
                Cell hotCell = e.Grid.HitTests.CellTest(e.Grid.PointToClient(Cursor.Position));
                if (Equals(e.Cell, hotCell) || Equals(e.Cell, mirorredCell))
                {
                    e.PaintAll();
                    e.Handled = true;
                    e.Render.DrawFrame(Color.Black, e.Cell.VirtualBounds, ButtonBorderStyle.Solid, e.Graphics);
                }
            }
            else
            {
                e.Appearance.BackColor = Color.White;
            }
        }

        static void InvalidateCell(Cell cell)
        {
            if (cell != null)
            {
                cell.Invalidate();
                if (cell.Row != null)
                {
                    cell.Row.Invalidate();
                }
                if (cell.Column != null)
                {
                    cell.Column.InvalidateCells();
                }
            }
        }

        Cell GetMirroredCell(Cell cell)
        {
            if (cell != null && cell.Row != null && cell.Column != null && cell.Row.DataObject != null)
            {
                string c1 = ((CurrencyRate)cell.Row.DataObject).MainCurrency;
                string c2 = cell.Column.Id;
                Row mirroredRow = grid.DataObjects.FindFirstRow(Provider.Instance.CurrencyRates[c2]);
                return mirroredRow != null ? mirroredRow[c1] : null;
            }
            return null;
        }

        private void grid_CellLeave(object sender, GridCellEventArgs e)
        {
            InvalidateCell(e.Cell);
            InvalidateCell(GetMirroredCell(e.Cell));
        }

        private void grid_CellEnter(object sender, GridCellEventArgs e)
        {
            InvalidateCell(e.Cell);
            InvalidateCell(GetMirroredCell(e.Cell));
        }
    }
}